"use strict";

// src/internal/index.ts
var context = {
  appContext: null
};
var OverlayMetaKey = Symbol("__imperative_overlay_key");

// src/meta.ts
import { getCurrentInstance, inject, onMounted, provide, ref, watch } from "vue-demi";
import delay from "delay";
import noop from "lodash/noop";
import { useVModel } from "@vueuse/core";
function useOverlayMeta(options = {}) {
  const { animation = 0, immediate = true, model = "visible" } = options;
  const defaultMeta = getTemplateMeta(model);
  const meta = inject(OverlayMetaKey, defaultMeta) || defaultMeta;
  !meta.isTemplate && watch(meta.visible, async () => {
    var _a;
    if (meta.visible.value)
      return void 0;
    if (animation > 0)
      await delay(animation);
    (_a = meta.vanish) == null ? void 0 : _a.call(meta);
  });
  if (immediate)
    onMounted(() => meta.visible.value = true);
  provide(OverlayMetaKey, null);
  return meta;
}
function getTemplateMeta(model) {
  const instance = getCurrentInstance();
  const visible = instance ? useVModel(instance.props, model) : ref(false);
  const cancel = (value) => {
    visible.value = false;
    instance == null ? void 0 : instance.emit("cancel", value);
  };
  const confirm = (value) => {
    visible.value = false;
    instance == null ? void 0 : instance.emit("confirm", value);
  };
  return {
    cancel,
    confirm,
    vanish: noop,
    visible,
    isTemplate: true
  };
}

// src/transform.ts
import { provide as provide2, ref as ref2 } from "vue";

// src/helper/render.ts
import { createVNode, defineComponent, h, render } from "vue";
var renderInstance = (component, props, options = {}) => {
  const vanish = () => {
    render(null, container);
  };
  const Provider = defineComponent({
    name: component.name,
    setup() {
      var _a;
      (_a = options.setup) == null ? void 0 : _a.call(options, vnode, vanish);
    },
    render() {
      return h(component, props);
    }
  });
  const container = document.createElement("div");
  const vnode = createVNode(Provider, props);
  if (context.appContext)
    vnode.appContext = context.appContext;
  if (options.appContext)
    vnode.appContext = options.appContext;
  render(vnode, container);
  if (container.firstElementChild) {
    const root = options.root || document.body;
    root.append(container.firstElementChild);
  }
  return { vanish, vnode };
};

// src/transform.ts
function createOverlay(component) {
  const executor = (props, promiser, mountOptions) => {
    const { reject, resolve, promise } = promiser;
    renderInstance(component, props, {
      appContext: mountOptions == null ? void 0 : mountOptions.appContext,
      root: mountOptions == null ? void 0 : mountOptions.root,
      setup: (vnode, _vanish) => {
        const visible = ref2(false);
        function cancel(value) {
          reject == null ? void 0 : reject(value);
          visible.value = false;
        }
        function confirm(value) {
          resolve == null ? void 0 : resolve(value);
          visible.value = false;
        }
        function vanish() {
          _vanish == null ? void 0 : _vanish();
          reject == null ? void 0 : reject();
        }
        promise.confirm = confirm;
        promise.cancel = cancel;
        provide2(OverlayMetaKey, {
          cancel,
          confirm,
          vanish,
          visible,
          vnode
        });
      }
    });
  };
  const caller = (props, options) => {
    const promiser = createPromiser();
    const notAllowedError = function() {
      throw new Error("createOverlay - Error: It is not allowed to call confirm and cancel externally immediately, please wait for the component to render");
    };
    promiser.promise.confirm = notAllowedError;
    promiser.promise.cancel = notAllowedError;
    executor(props, promiser, options);
    return promiser.promise;
  };
  return caller;
}
function createPromiser() {
  let resolve;
  let reject;
  const promise = new Promise((_resolve, _reject) => {
    resolve = _resolve;
    reject = _reject;
  });
  return { promise, reject, resolve };
}

// src/utils.ts
function executeOverlay(component, options) {
  return createOverlay(component)(options == null ? void 0 : options.props, options);
}

// src/index.ts
var install = (app) => {
  context.appContext = app._context;
};
var unoverlay = { install };
var src_default = unoverlay;
export {
  createOverlay,
  src_default as default,
  executeOverlay,
  install,
  useOverlayMeta
};
//# sourceMappingURL=index.esm.js.map
